# Module captain-pool/esrgan-tf2/1
Enhanced Super Resolution GAN (Wang et. al.)[1] for image super resolution.
Produces x4 Super Resolution Image from images of {Height, Width} >=64.
Works best on Bicubically downsampled images.\
(*This is because, the model is originally trained on Bicubically Downsampled DIV2K Dataset*)

<!-- asset-path: https://github.com/captain-pool/GSOC/releases/download/1.0.0/esrgan.tar.gz -->
<!-- module-type: image-super-resolution -->
<!-- network-architecture: GAN -->
<!-- dataset: DIV2K -->
<!-- language: en -->
<!-- fine-tunable: true -->
<!-- format: saved_model_2 -->
<!-- license: MIT -->

[![Open Colab notebook]](https://colab.research.google.com/github/tensorflow/hub/blob/master/examples/colab/image_enhancing.ipynb)

### NOTE
- The image must be a float32 image, converted using `tf.cast(image, tf.float32)`.
- The image must be of 4 dimensions, `[batch_size, height, width, 3]`.
  To perform super resolution on a single image, use `tf.expand_dims(image, 0)` to add
  the batch dimension.
- To display the image, don't forget to convert back to `uint8` using  
`tf.cast(tf.clip_by_value(image[index_of_image_to_display], 0, 255), tf.uint8)`

### Fine Tuning
This Saved Model only ships the Generator part of the complete model. So, to train it
adversarially on a different set of Images  with different form of Down Sampling Function,
needs the user to define a new discriminator (since the generator is pretrained, it is
preferable to keep a higher learning rate of the discriminator, during the initial steps)
and train the model adversarially. Refer to the training codes for reference [2].

### Example Use

The SavedModel can be loaded directly
```python3
import tensorflow_hub as hub
import tensorflow as tf
model = hub.load("https://tfhub.dev/captain-pool/esrgan-tf2/1")
# To add an extra dimension for batch, use tf.expand_dims()
low_resolution_image = load_image() # Low Resolution Image of shape [batch_size, height, width, 3]
low_resolution_image = tf.cast(low_resolution_image, tf.float32)
super_resolution = model(low_resolution_image) # Perform Super Resolution here
```
or with `hub.KerasLayer` for retraining

```python3
import tensorflow_hub as hub
import tensorflow as tf
generator = tf.keras.models.Sequential([
    hub.KerasLayer("https://tfhub.dev/captain-pool/esrgan-tf2/1", trainable=True),
    tf.keras.layers.Conv2D(filters=3, kernel_size=[1, 1], strides=[1, 1])
])
```

References
--------------
[1] [ESRGAN: Enhanced Super-Resolution Generative Adversarial Networks](https://arxiv.org/abs/1809.00219)
[2] [TF 2.0 Implementation of ESRGAN](https://github.com/captain-pool/GSOC/tree/master/E2_ESRGAN)
